@@ -25,14 +25,14 @@ DEBUG_BUILD=0
 ASAN_BUILD=0
 DEPLOY=1
 ENABLE_FREI0R=1
-FREI0R_HEAD=0
-FREI0R_REVISION="36e7da5d9e1f8b8cac9e97e204db1c5834fee580"
+FREI0R_HEAD=1
+FREI0R_REVISION=
 SUBDIRS=
 ENABLE_MOVIT=1
 MOVIT_HEAD=0
 MOVIT_REVISION="origin/shotcut-opengl3"
 FFMPEG_HEAD=0
-FFMPEG_REVISION="origin/release/7.0"
+FFMPEG_REVISION="origin/release/7.1"
 FFMPEG_ADDITIONAL_OPTIONS=
 MLT_HEAD=1
 MLT_REVISION=
@@ -42,11 +42,11 @@ SHOTCUT_REVISION=
 SHOTCUT_VERSION=$(date '+%y.%m.%d')
 ENABLE_BIGSH0T=1
 BIGSH0T_HEAD=0
-BIGSH0T_REVISION="8fe56f6d4e"
+BIGSH0T_REVISION="710e734512b8975722f0c41f793f28c51a3d2b88"
 DAV1D_HEAD=0
-DAV1D_REVISION="1.4.2"
+DAV1D_REVISION="1.5.0"
 AOM_HEAD=0
-AOM_REVISION="v3.8.0"
+AOM_REVISION="v3.11.0"
 ENABLE_VMAF=1
 VMAF_HEAD=0
 VMAF_REVISION="v3.0.0"
@@ -56,14 +56,25 @@ GLAXNIMATE_REVISION="origin/v0.5.4"
 ENABLE_GOPRO2GPX=1
 ENABLE_OPENCV=1
 OPENCV_HEAD=0
-OPENCV_REVISION="4.9.0"
+OPENCV_REVISION="origin/4.x"
+OPENCV_CONTRIB_HEAD=0
+OPENCV_CONTRIB_REVISION="4.11.0"
 ENABLE_LIBSPATIALAUDIO=1
-LIBSPATIALAUDIO_HEAD=1
-LIBSPATIALAUDIO_REVISION=
+LIBSPATIALAUDIO_HEAD=0
+LIBSPATIALAUDIO_REVISION="origin/patch-1"
 ENABLE_LADSPA=1
 LADSPA_HEAD=0
 LADSPA_REVISION="origin/shotcut"
 MLT_DISABLE_SOX=0
+ENABLE_WHISPERCPP=1
+WHISPERCPP_HEAD=0
+WHISPERCPP_REVISION="v1.7.4"
+NV_CODEC_REVISION="sdk/12.0"
+
+PYTHON_VERSION=$(python3 --version | awk '{split($2, parts, "."); print parts[1] "." parts[2]}')
+PYTHON_VERSION_DLL=$(python3 --version | awk '{split($2, parts, "."); print parts[1]parts[2]}')
+
+QT_VERSION_X64="6.9.1"
 
 ################################################################################
 # Location of config file - if not overridden on command line
@@ -160,6 +171,9 @@ function to_key {
     bigsh0t)
       echo 8
     ;;
+    whisper.cpp)
+      echo 9
+    ;;
     dav1d)
       echo 10
     ;;
@@ -336,7 +350,7 @@ function set_globals {
     QT_LIB_DIR="$QTDIR/lib"
     QT_SHARE_DIR="$QTDIR/share/qt6"
   else
-    export QTDIR="$HOME/Qt/6.7.1/mingw_64"
+    export QTDIR="$HOME/Qt/$QT_VERSION_X64/mingw_64"
     QT_INCLUDE_DIR="$QTDIR/include"
     QT_LIB_DIR="$QTDIR/lib"
     QT_SHARE_DIR="$QTDIR"
@@ -394,6 +408,9 @@ function set_globals {
     if test "$ENABLE_LADSPA" = 1 && test "$LADSPA_HEAD" = 1 -o "$LADSPA_REVISION" != ""; then
         SUBDIRS="ladspa-swh $SUBDIRS"
     fi
+    if test "$ENABLE_WHISPERCPP" = 1  && test "$WHISPERCPP_HEAD" = 1 -o "$WHISPERCPP_REVISION" != ""; then
+        SUBDIRS="whisper.cpp $SUBDIRS"
+    fi
     SUBDIRS="$SUBDIRS mlt shotcut"
   fi
 
@@ -428,14 +445,15 @@ function set_globals {
   REPOLOCS[7]="https://github.com/GPUOpen-LibrariesAndSDKs/AMF.git"
 #  REPOLOCS[8]="https://bitbucket.org/dandennedy/bigsh0t.git"
   REPOLOCS[8]="https://bitbucket.org/leo_sutic/bigsh0t.git"
-  REPOLOCS[10]="https://code.videolan.org/videolan/dav1d.git"
+  REPOLOCS[9]="https://github.com/ggerganov/whisper.cpp.git"
+  REPOLOCS[10]="https://github.com/videolan/dav1d.git"
   REPOLOCS[11]="https://aomedia.googlesource.com/aom"
   REPOLOCS[12]="https://github.com/Netflix/vmaf.git"
   REPOLOCS[13]="https://gitlab.com/ddennedy/glaxnimate.git"
   REPOLOCS[14]="https://github.com/ddennedy/gopro2gpx.git"
   REPOLOCS[15]="https://github.com/opencv/opencv.git"
   REPOLOCS[16]="https://github.com/opencv/opencv_contrib.git"
-  REPOLOCS[17]="https://github.com/videolabs/libspatialaudio.git"
+  REPOLOCS[17]="https://github.com/ddennedy/libspatialaudio.git"
 
   # REPOTYPE Array holds the repo types. (Yes, this might be redundant, but easy for me)
   REPOTYPES[0]="git"
@@ -482,13 +500,16 @@ function set_globals {
   if test 0 = "$LADSPA_HEAD" -a "$LADSPA_REVISION" ; then
     REVISIONS[5]="$LADSPA_REVISION"
   fi
-  REVISIONS[6]="sdk/12.0" # nv-codec-headers
+  REVISIONS[6]="$NV_CODEC_REVISION"
   REVISIONS[7]="" # AMF
   REVISIONS[8]=""
   if test 0 = "$BIGSH0T_HEAD" -a "$BIGSH0T_REVISION" ; then
     REVISIONS[8]="$BIGSH0T_REVISION"
   fi
   REVISIONS[9]=""
+  if test 0 = "$WHISPERCPP_HEAD" -a "$WHISPERCPP_REVISION" ; then
+    REVISIONS[9]="$WHISPERCPP_REVISION"
+  fi
   REVISIONS[10]=""
   if test 0 = "$DAV1D_HEAD" -a "$DAV1D_REVISION" ; then
     REVISIONS[10]="$DAV1D_REVISION"
@@ -511,12 +532,12 @@ function set_globals {
     REVISIONS[15]="$OPENCV_REVISION"
   fi
   REVISIONS[16]=""
-  if test 0 = "$OPENCV_HEAD" -a "$OPENCV_REVISION" ; then
-    REVISIONS[16]="$OPENCV_REVISION"
+  if test 0 = "$OPENCV_CONTRIB_HEAD" -a "$OPENCV_CONTRIB_REVISION" ; then
+    REVISIONS[16]="$OPENCV_CONTRIB_REVISION"
   fi
   REVISIONS[17]=""
   if test 0 = "$LIBSPATIALAUDIO_HEAD" -a "$LIBSPATIALAUDIO_REVISION" ; then
-    REVISIONS[17]="$LIBSPATIALAUDIO_REVISION_REVISION"
+    REVISIONS[17]="$LIBSPATIALAUDIO_REVISION"
   fi
 
   # Figure out the number of cores in the system. Used both by make and startup script
@@ -562,7 +583,7 @@ function set_globals {
 
   #####
   # mlt
-  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DMOD_GDK=OFF -DMOD_GLAXNIMATE_QT6=ON -DMOD_QT=OFF -DMOD_QT6=ON -DMOD_SDL1=OFF -DUSE_VST2=OFF"
+  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DMOD_GDK=OFF -DMOD_GLAXNIMATE_QT6=ON -DMOD_QT=OFF -DMOD_QT6=ON -DMOD_SDL1=OFF -DUSE_VST2=OFF -DMOD_SPATIALAUDIO=ON"
   [ "$ENABLE_OPENCV" = "1" ] && CONFIG[1]="${CONFIG[1]} -DMOD_OPENCV=ON"
   if [ "$DEBUG_BUILD" = "1" ]; then
     CONFIG[1]="${CONFIG[1]} -DCMAKE_BUILD_TYPE=RelWithDebInfo"
@@ -625,6 +646,19 @@ function set_globals {
   BUILD[8]="ninja -j $MAKEJ"
   INSTALL[8]="install -p -c *.dll "$FINAL_INSTALL_DIR"/lib/frei0r-1"
 
+  #####
+  # whisper.cpp
+  CONFIG[9]="cmake -B build -G Ninja -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR $CMAKE_DEBUG_FLAG -D BUILD_SHARED_LIBS=ON -D GGML_NATIVE=OFF -D WHISPER_BUILD_SERVER=OFF -D WHISPER_BUILD_TESTS=OFF"
+  if [ "$TARGET_ARCH" = "clangarm64" ]; then
+    CONFIG[9]="${CONFIG[9]} -D GGML_BLAS=OFF"
+  else
+    CONFIG[9]="${CONFIG[9]} -D GGML_AVX2=OFF -D GGML_BLAS=ON -D GGML_BLAS_VENDOR=OpenBLAS"
+  fi
+  CFLAGS_[9]=$CFLAGS
+  LDFLAGS_[9]=$LDFLAGS
+  BUILD[9]="ninja -C build -j $MAKEJ"
+  INSTALL[9]="install_whispercpp"
+
   #####
   # dav1d
   CONFIG[10]="meson setup builddir --prefix=$FINAL_INSTALL_DIR --libdir=$FINAL_INSTALL_DIR/lib"
@@ -664,7 +698,7 @@ function set_globals {
   # glaxnimate
   CONFIG[13]="cmake -G Ninja -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -D Python3_FIND_REGISTRY=NEVER"
   if [ "$TARGET_ARCH" = "clangarm64" ]; then
-    CONFIG[13]="${CONFIG[13]} -D CMAKE_PREFIX_PATH=$FINAL_INSTALL_DIR"
+    CONFIG[13]="${CONFIG[13]} -D CMAKE_CXX_FLAGS=-DPYBIND11_EXPORT_EXCEPTION= -D CMAKE_PREFIX_PATH=$FINAL_INSTALL_DIR"
   else
     CONFIG[13]="${CONFIG[13]} -D CMAKE_PREFIX_PATH=$QTDIR"
   fi
@@ -691,7 +725,7 @@ function set_globals {
   CONFIG[15]="cmake -B build -G Ninja -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -D BUILD_LIST=tracking -D OPENCV_GENERATE_PKGCONFIG=YES -D OPENCV_EXTRA_MODULES_PATH=../opencv_contrib/modules -D WITH_OPENMP=ON $CMAKE_DEBUG_FLAG"
   CFLAGS_[15]="$CFLAGS"
   LDFLAGS_[15]="$LDFLAGS"
-  BUILD[15]="build_opencv"
+  BUILD[15]="ninja -C build -j $MAKEJ"
   INSTALL[15]="ninja -C build install"
 
   #####
@@ -708,11 +742,6 @@ function build_dav1d {
   cmd ninja -C builddir -j $MAKEJ || cmd ninja -C builddir -j $MAKEJ
 }
 
-function build_opencv {
-  [ grep M_PI 3rdparty/carotene/include/carotene/definitions.hpp ] || printf >>3rdparty/carotene/include/carotene/definitions.hpp '#ifndef M_PI\n#define M_PI (3.14159265358979323846)\n#endif'
-  cmd ninja -C build -j $MAKEJ
-}
-
 function install_amf {
   cmd rm -rf Thirdparty
   cmd mkdir -p "$FINAL_INSTALL_DIR/include/AMF"
@@ -744,6 +773,10 @@ function install_spatialaudio {
   cmd sed -i "s,-I/${TARGET_ARCH}/include,," "$FINAL_INSTALL_DIR"/lib/pkgconfig/spatialaudio.pc
 }
 
+function install_whispercpp {
+  cmd ninja -C build install
+  cmd install -p -c build/bin/whisper-cli.exe $FINAL_INSTALL_DIR/bin
+}
 
 ######################################################################
 # FEEDBACK FUNCTIONS
@@ -1179,6 +1212,7 @@ function deploy
     cmd mv bin/ffplay.exe .
     cmd mv bin/ffprobe.exe .
     cmd mv bin/glaxnimate.exe .
+    cmd mv bin/whisper-cli.exe .
     cmd rm -rf bin include etc man manifest src *.txt
     cmd rm lib/*
     cmd rm -rf lib/cmake lib/pkgconfig lib/gdk-pixbuf-2.0 lib/glib-2.0 lib/gtk-2.0
@@ -1193,7 +1227,7 @@ function deploy
 
   log Copying some libs from Qt
   if [ "$DEBUG_BUILD" != "1" -o "$SDK" = "1" ]; then
-    cmd cp -p "$QTDIR"/bin/Qt6{Charts,Core,Core5Compat,Gui,Multimedia,Network,OpenGL,OpenGLWidgets,Qml,QmlModels,QmlWorkerScript,Quick,QuickControls2*,QuickDialogs2,QuickDialogs2QuickImpl,QuickDialogs2Utils,QuickLayouts,QuickTemplates2,QuickWidgets,Sql,Svg,SvgWidgets,UiTools,Widgets,Xml}.dll .
+    cmd cp -p "$QTDIR"/bin/Qt6{Charts,Core,Core5Compat,Gui,Multimedia,Network,OpenGL,OpenGLWidgets,Qml,QmlMeta,QmlModels,QmlWorkerScript,Quick,QuickControls2*,QuickDialogs2,QuickDialogs2QuickImpl,QuickDialogs2Utils,QuickLayouts,QuickTemplates2,QuickWidgets,Sql,Svg,SvgWidgets,UiTools,Widgets,Xml}.dll .
   fi
 
   for lib in *.dll; do
@@ -1209,12 +1243,12 @@ function deploy
   done
 
   log Copying some DLLs and python libraries for Glaxnimate
-  cmd cp -p /${TARGET_ARCH}/bin/libpython3.11.dll python311.dll
+  cmd cp -p /${TARGET_ARCH}/bin/libpython${PYTHON_VERSION}.dll python${PYTHON_VERSION_DLL}.dll
   cmd cp -p "$SOURCE_DIR"/glaxnimate/external/Qt-Color-Widgets/libQtColorWidgets.dll .
   cmd mkdir -p share/glaxnimate/glaxnimate/pythonhome/lib/python
-  cmd cp -r /${TARGET_ARCH}/lib/python3.11/*.py \
-            /${TARGET_ARCH}/lib/python3.11/lib-dynload/* \
-            /${TARGET_ARCH}/lib/python3.11/{json,collections,encodings,logging,urllib} \
+  cmd cp -r /${TARGET_ARCH}/lib/python${PYTHON_VERSION}/*.py \
+            /${TARGET_ARCH}/lib/python${PYTHON_VERSION}/lib-dynload/* \
+            /${TARGET_ARCH}/lib/python${PYTHON_VERSION}/{json,collections,encodings,logging,urllib} \
       share/glaxnimate/glaxnimate/pythonhome/lib/python
 
   log Copying some libs from mlt-prebuilt
