@@ -27,8 +27,8 @@ DEBUG_BUILD=0
 ASAN_BUILD=0
 CREATE_STARTUP_SCRIPT=1
 ENABLE_FREI0R=1
-FREI0R_HEAD=0
-FREI0R_REVISION="36e7da5d9e1f8b8cac9e97e204db1c5834fee580"
+FREI0R_HEAD=1
+FREI0R_REVISION=
 ENABLE_MOVIT=1
 SUBDIRS=
 MOVIT_HEAD=0
@@ -38,15 +38,15 @@ X264_REVISION="origin/stable"
 X265_HEAD=0
 X265_REVISION="origin/stable"
 LIBVPX_HEAD=0
-LIBVPX_REVISION="v1.13.1"
+LIBVPX_REVISION="v1.15.0"
 LIBOPUS_HEAD=0
-LIBOPUS_REVISION="v1.4"
+LIBOPUS_REVISION="v1.5.2"
 ENABLE_SWH_PLUGINS=1
 FFMPEG_HEAD=0
-FFMPEG_REVISION="origin/release/7.0"
+FFMPEG_REVISION="origin/release/7.1"
 FFMPEG_SUPPORT_H264=1
 FFMPEG_SUPPORT_H265=1
-FFMPEG_SUPPORT_JACK=1
+FFMPEG_SUPPORT_JACK=0
 FFMPEG_SUPPORT_LIBVPX=1
 FFMPEG_SUPPORT_THEORA=1
 FFMPEG_SUPPORT_MP3=1
@@ -73,19 +73,19 @@ SHOTCUT_REVISION=
 SHOTCUT_VERSION=$(date '+%y.%m.%d')
 ENABLE_RUBBERBAND=1
 RUBBERBAND_HEAD=0
-RUBBERBAND_REVISION="v3.3.0"
+RUBBERBAND_REVISION="v4.0.0"
 ENABLE_BIGSH0T=1
 BIGSH0T_HEAD=0
-BIGSH0T_REVISION="8fe56f6d4e"
+BIGSH0T_REVISION="2.7"
 ENABLE_ZIMG=1
 ZIMG_HEAD=0
 ZIMG_REVISION="51c3c7f750c2af61955377faad56e3ba1b03589f"
 DAV1D_HEAD=0
-DAV1D_REVISION="1.4.2"
+DAV1D_REVISION="1.5.0"
 AOM_HEAD=0
-AOM_REVISION="v3.8.0"
+AOM_REVISION="v3.11.0"
 SVTAV1_HEAD=0
-SVTAV1_REVISION="v2.1.0"
+SVTAV1_REVISION="v3.0.2"
 VMAF_HEAD=0
 VMAF_REVISION="v3.0.0"
 ENABLE_GLAXNIMATE=1
@@ -94,19 +94,25 @@ GLAXNIMATE_REVISION="origin/v0.5.4"
 ENABLE_GOPRO2GPX=1
 ENABLE_OPENCV=1
 OPENCV_HEAD=0
-OPENCV_REVISION="4.9.0"
+OPENCV_REVISION="4.11.0"
 ENABLE_LIBWEBP=1
 LIBWEBP_HEAD=0
-LIBWEBP_REVISION="v1.3.2"
+LIBWEBP_REVISION="v1.5.0"
 ENABLE_LIBSPATIALAUDIO=1
-LIBSPATIALAUDIO_HEAD=1
-LIBSPATIALAUDIO_REVISION=
+LIBSPATIALAUDIO_HEAD=0
+LIBSPATIALAUDIO_REVISION="origin/patch-1"
+ENABLE_WHISPERCPP=1
+WHISPERCPP_HEAD=0
+WHISPERCPP_REVISION="v1.7.4"
+NV_CODEC_REVISION="sdk/12.0"
+MFX_DISPATCH_REVISION="1.25"
 
 PYTHON_VERSION_DEFAULT=3.8
 PYTHON_VERSION_DARWIN=3.11
+PYTHON_VERSION=$(python3 --version | awk '{split($2, parts, "."); print parts[1] "." parts[2]}')
 
 QT_VERSION_DEFAULT=6.4.3
-QT_VERSION_DARWIN=6.7.1
+QT_VERSION_DARWIN=6.9.1
 
 # QT_INCLUDE_DIR="$(pkg-config --variable=prefix QtCore)/include"
 QT_INCLUDE_DIR=${QTDIR:+${QTDIR}/include}
@@ -214,6 +220,9 @@ function to_key {
     ladspa)
       echo 8
     ;;
+    OpenBLAS)
+      echo 9
+    ;;
     vid.stab)
       echo 10
     ;;
@@ -268,6 +277,9 @@ function to_key {
     SVT-AV1)
       echo 29
     ;;
+    whisper.cpp)
+      echo 30
+    ;;
     *)
       echo UNKNOWN
     ;;
@@ -514,6 +526,10 @@ function set_globals {
     if test "$ENABLE_LIBSPATIALAUDIO" = 1  && test "$LIBSPATIALAUDIO_HEAD" = 1 -o "$LIBSPATIALAUDIO_REVISION" != ""; then
         SUBDIRS="libspatialaudio $SUBDIRS"
     fi
+    if test "$ENABLE_WHISPERCPP" = 1  && test "$WHISPERCPP_HEAD" = 1 -o "$WHISPERCPP_REVISION" != ""; then
+        SUBDIRS="whisper.cpp $SUBDIRS"
+        [ "$TARGET_OS" != "Darwin" ] && SUBDIRS="OpenBLAS $SUBDIRS"
+    fi
   fi
 
   if [ "$DEBUG_BUILD" = "1" ]; then
@@ -545,9 +561,10 @@ function set_globals {
   REPOLOCS[3]="https://github.com/mirror/x264.git"
   REPOLOCS[4]="https://chromium.googlesource.com/webm/libvpx.git"
   REPOLOCS[5]="https://github.com/ddennedy/movit.git"
-  REPOLOCS[6]="https://github.com/videolabs/libspatialaudio.git"
+  REPOLOCS[6]="https://github.com/ddennedy/libspatialaudio.git"
   REPOLOCS[7]="https://github.com/mltframework/shotcut.git"
   REPOLOCS[8]="https://github.com/swh/ladspa.git"
+  REPOLOCS[9]="https://github.com/OpenMathLib/OpenBLAS.git"
   REPOLOCS[10]="https://github.com/georgmartius/vid.stab.git"
   REPOLOCS[12]="https://github.com/xiph/opus.git"
   REPOLOCS[13]="https://github.com/videolan/x265"
@@ -566,6 +583,7 @@ function set_globals {
   REPOLOCS[27]="https://github.com/opencv/opencv_contrib.git"
   REPOLOCS[28]="https://github.com/webmproject/libwebp.git"
   REPOLOCS[29]="https://gitlab.com/AOMediaCodec/SVT-AV1.git"
+  REPOLOCS[30]="https://github.com/ggerganov/whisper.cpp.git"
 
   # REPOTYPE Array holds the repo types. (Yes, this might be redundant, but easy for me)
   REPOTYPES[0]="git"
@@ -596,6 +614,7 @@ function set_globals {
   REPOTYPES[27]="git"
   REPOTYPES[28]="git"
   REPOTYPES[29]="git"
+  REPOTYPES[30]="git"
 
   # And, set up the revisions
   REVISIONS[0]=""
@@ -625,13 +644,14 @@ function set_globals {
   fi
   REVISIONS[6]=""
   if test 0 = "$LIBSPATIALAUDIO_HEAD" -a "$LIBSPATIALAUDIO_REVISION" ; then
-    REVISIONS[6]="$LIBSPATIALAUDIO_REVISION_REVISION"
+    REVISIONS[6]="$LIBSPATIALAUDIO_REVISION"
   fi
   REVISIONS[7]=""
   if test 0 = "$SHOTCUT_HEAD" -a "$SHOTCUT_REVISION" ; then
     REVISIONS[7]="$SHOTCUT_REVISION"
   fi
   REVISIONS[8]=""
+  REVISIONS[9]=""
   REVISIONS[10]=""
   if test 0 = "$VIDSTAB_HEAD" -a "$VIDSTAB_REVISION" ; then
     REVISIONS[10]="$VIDSTAB_REVISION"
@@ -644,9 +664,9 @@ function set_globals {
   if test 0 = "$X265_HEAD" -a "$X265_REVISION" ; then
     REVISIONS[13]="$X265_REVISION"
   fi
-  REVISIONS[15]="sdk/12.0" # nv-codec-headers
+  REVISIONS[15]="$NV_CODEC_REVISION"
   REVISIONS[16]=""
-  REVISIONS[17]="1.25"
+  REVISIONS[17]="$MFX_DISPATCH_REVISION"
   REVISIONS[18]=""
   if test 0 = "$RUBBERBAND_HEAD" -a "$RUBBERBAND_REVISION" ; then
     REVISIONS[18]="$RUBBERBAND_REVISION"
@@ -692,6 +712,10 @@ function set_globals {
   if test 0 = "$SVTAV1_HEAD" -a "$SVTAV1_REVISION" ; then
     REVISIONS[29]="$SVTAV1_REVISION"
   fi
+  REVISIONS[30]=""
+  if test 0 = "$WHISPERCPP_HEAD" -a "$WHISPERCPP_REVISION" ; then
+    REVISIONS[30]="$WHISPERCPP_REVISION"
+  fi
 
   # Figure out the number of cores in the system. Used both by make and startup script
   if test "$TARGET_OS" = "Darwin"; then
@@ -800,14 +824,14 @@ function set_globals {
     BUILD[0]="build_ffmpeg_darwin"
     INSTALL[0]="install_ffmpeg_darwin"
   elif test "$TARGET_OS" = "Linux" ; then
-    CONFIG[0]="${CONFIG[0]} --enable-libxcb --enable-libpulse --prefix=$FINAL_INSTALL_DIR"
+    CONFIG[0]="${CONFIG[0]} --enable-libpulse --prefix=$FINAL_INSTALL_DIR"
     BUILD[0]="make -j$MAKEJ"
     INSTALL[0]="make install"
   fi
 
   #####
   # mlt
-  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DMOD_QT=OFF -DMOD_QT6=ON -DMOD_GLAXNIMATE_QT6=ON -DMOD_GDK=OFF -DMOD_SDL1=OFF -DUSE_VST2=OFF"
+  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DMOD_QT=OFF -DMOD_QT6=ON -DMOD_GLAXNIMATE_QT6=ON -DMOD_GDK=OFF -DMOD_SDL1=OFF -DUSE_VST2=OFF -DMOD_SPATIALAUDIO=ON"
   # Remember, if adding more of these, to update the post-configure check.
   [ "$ENABLE_OPENCV" = "1" ] && CONFIG[1]="${CONFIG[1]} -DMOD_OPENCV=ON"
   [ "$MLT_DISABLE_SOX" = "1" ] && CONFIG[1]="${CONFIG[1]} -DMOD_SOX=OFF"
@@ -920,6 +944,14 @@ function set_globals {
   LDFLAGS_[8]=$LDFLAGS
   INSTALL[8]="install_ladspa"
 
+  #####
+  # OpenBLAS
+  CONFIG[9]="cmake -G Ninja -B builddir -D BUILD_SHARED_LIBS=ON -D USE_THREAD=ON -D NUM_THREADS=64 -D USE_OPENMP=ON -D TARGET=CORE2 -D DYNAMIC_ARCH=ON -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR $CMAKE_DEBUG_FLAG"
+  CFLAGS_[9]="$ASAN_CFLAGS $CFLAGS"
+  LDFLAGS_[9]="$ASAN_LDFLAGS $LDFLAGS"
+  BUILD[9]="ninja -C builddir -j $MAKEJ"
+  INSTALL[9]="ninja -C builddir install"
+
   #####
   # vid.stab
   CONFIG[10]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_INSTALL_LIBDIR=lib $CMAKE_DEBUG_FLAG"
@@ -1047,7 +1079,7 @@ function set_globals {
   CONFIG[24]="cmake -G Ninja -DCMAKE_PREFIX_PATH='$FINAL_INSTALL_DIR;$QTDIR' -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR $CMAKE_DEBUG_FLAG"
   if [ "$TARGET_OS" = "Darwin" ]; then
     CONFIG[24]="${CONFIG[24]} -DCMAKE_OSX_ARCHITECTURES='arm64;x86_64'"
-    CONFIG[24]="${CONFIG[24]} -DPython3_EXECUTABLE=/opt/local/Library/Frameworks/Python.framework/Versions/${PYTHON_VERSION_DARWIN}/bin/python${PYTHON_VERSION_DARWIN}"
+    CONFIG[24]="${CONFIG[24]} -DPython3_EXECUTABLE=/opt/local/Library/Frameworks/Python.framework/Versions/${PYTHON_VERSION}/bin/python${PYTHON_VERSION_DARWIN}"
   fi
   CFLAGS_[24]="$ASAN_CFLAGS $CFLAGS"
   LDFLAGS_[24]="$ASAN_LDFLAGS $LDFLAGS"
@@ -1089,6 +1121,22 @@ function set_globals {
   LDFLAGS_[29]=$LDFLAGS
   BUILD[29]="ninja -C build -j $MAKEJ"
   INSTALL[29]="ninja -C build install"
+  
+  #####
+  # whisper.cpp
+  CONFIG[30]="cmake -B build -G Ninja -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR $CMAKE_DEBUG_FLAG -D BUILD_SHARED_LIBS=ON -D GGML_NATIVE=OFF -D GGML_AVX2=OFF -D WHISPER_BUILD_SERVER=OFF -D WHISPER_BUILD_TESTS=OFF"
+  [ "$TARGET_OS" = "Darwin" ] && CONFIG[30]="${CONFIG[30]} -D CMAKE_OSX_ARCHITECTURES='arm64;x86_64'"
+  if test "$TARGET_OS" = "Darwin" ; then
+    CONFIG[30]="${CONFIG[30]} -D CMAKE_OSX_ARCHITECTURES='arm64;x86_64'"
+    CONFIG[30]="${CONFIG[30]} -D OpenMP_C_FLAGS=-I/opt/local/include/libomp -D OpenMP_CXX_FLAGS=-I/opt/local/include/libomp -D OpenMP_C_LIB_NAMES=libomp -D OpenMP_CXX_LIB_NAMES=libomp -D OpenMP_libomp_LIBRARY=omp"
+    LDFLAGS_[30]="$LDFLAGS -L /opt/local/lib/libomp"
+  else
+    CONFIG[30]="${CONFIG[30]} -D GGML_BLAS=ON -D GGML_BLAS_VENDOR=OpenBLAS"
+    CFLAGS_[30]=$CFLAGS
+    LDFLAGS_[30]=$LDFLAGS
+  fi
+  BUILD[30]="ninja -C build -j $MAKEJ"
+  INSTALL[30]="install_whispercpp"
 }
 
 function build_ffmpeg_darwin {
@@ -1200,6 +1248,12 @@ EOF
   cmd ninja install
 }
 
+function install_whispercpp {
+  cmd ninja -C build install
+  cmd mkdir -p $FINAL_INSTALL_DIR/bin
+  cmd install -p -c build/bin/whisper-cli $FINAL_INSTALL_DIR/bin
+}
+
 ######################################################################
 # FEEDBACK FUNCTIONS
 ######################################################################
@@ -1579,7 +1633,7 @@ function configure_compile_install_subproject {
     elif [ "libspatialaudio" = "$1" ]; then
       replace_rpath spatialaudio
     elif [ "SVT-AV1" = "$1" ]; then
-    replace_rpath SvtAv1Enc
+      replace_rpath SvtAv1Enc
     elif [ "vid.stab" = "$1" ]; then
       cmd sed -e 's/-fopenmp//' -i .bak "$FINAL_INSTALL_DIR/lib/pkgconfig/vidstab.pc"
     fi
@@ -1832,7 +1886,7 @@ function deploy_mac
 
   log Copying supplementary executables
   cmd mkdir -p MacOS 2>/dev/null
-  cmd cp -a "$FINAL_INSTALL_DIR"/bin/{melt,ffmpeg,ffplay,ffprobe,glaxnimate,gopro2gpx} MacOS
+  cmd cp -a "$FINAL_INSTALL_DIR"/bin/{melt,ffmpeg,ffplay,ffprobe,glaxnimate,gopro2gpx,whisper-cli} MacOS
   cmd mkdir -p Frameworks 2>/dev/null
   cmd cp -p ../../lib/libCuteLogger.dylib Frameworks
   for exe in MacOS/Shotcut MacOS/melt MacOS/ffmpeg MacOS/ffplay MacOS/ffprobe MacOS/glaxnimate; do
@@ -1842,7 +1896,16 @@ function deploy_mac
     cmd install_name_tool -delete_rpath "$QTDIR/lib" "$exe" 2> /dev/null
     cmd install_name_tool -add_rpath "@executable_path/../Frameworks" "$exe"
   done
-  cmd cp -p "$FINAL_INSTALL_DIR"/lib/libaom.2.dylib Frameworks
+  cmd cp -p /opt/local/lib/libaom.3.dylib Frameworks
+
+  # whisper.cpp
+  cmd cp -p "$FINAL_INSTALL_DIR"/lib/libwhisper.1.dylib Frameworks
+  cmd cp -p "$FINAL_INSTALL_DIR"/lib/libggml*.dylib Frameworks
+  fixlibs MacOS/whisper-cli
+  log fixing rpath of executable "whisper-cli"
+  cmd install_name_tool -delete_rpath "$SOURCE_DIR"/whisper.cpp/build/src MacOS/whisper-cli 2> /dev/null
+  cmd install_name_tool -delete_rpath "$SOURCE_DIR"/whisper.cpp/build/ggml/src MacOS/whisper-cli 2> /dev/null
+  cmd install_name_tool -add_rpath "@executable_path/../Frameworks" MacOS/whisper-cli
 
   # MLT plugins
   log Copying MLT plugins
@@ -1912,6 +1975,10 @@ function deploy_mac
   cmd install -d lib
   cmd cp -pLR /opt/local/Library/Frameworks/Python.framework/Versions/${PYTHON_VERSION_DARWIN}/lib/python${PYTHON_VERSION_DARWIN} lib
 
+  # Whisper.cpp models
+  log Copying Whisper.cpp models
+  cmd cp -a "$FINAL_INSTALL_DIR"/share/shotcut/whisper_models Resources/shotcut
+
   log Fixing rpath in libraries
   cmd find . -name '*.dylib' -exec sh -c "install_name_tool -delete_rpath \"/opt/local/lib/libomp\" {} 2> /dev/null" \;
   cmd find . -name '*.dylib' -exec sh -c "install_name_tool -delete_rpath \"$FINAL_INSTALL_DIR/lib\" {} 2> /dev/null" \;
@@ -1941,7 +2008,7 @@ function deploy_mac
     cmd cp -a "$FINAL_INSTALL_DIR"/lib/pkgconfig Shotcut/Contents/Frameworks/lib
     log Symlinking libs
     pushd Shotcut/Contents/Frameworks
-    for lib in avcodec avdevice avfilter avformat avutil epoxy mlt++-7 mlt-7 movit mp3lame opus postproc swresample swscale vidstab x264 x265; do
+    for lib in avcodec avdevice avfilter avformat avutil epoxy mlt++-7 mlt-7 movit mp3lame opus postproc swresample swscale vidstab whisper x264 x265; do
       dylib=$(ls lib$lib.*.dylib | head -n 1)
       cmd ln -sf $dylib lib$lib.dylib
     done
@@ -1987,6 +2054,8 @@ function create_startup_script {
   fi
 
   cmd cp -pLR /usr/lib/python${PYTHON_VERSION_DEFAULT} "$FINAL_INSTALL_DIR"/lib
+  cmd cp -p /usr/lib/x86_64-linux-gnu/libcrypto.so.1.1 "$FINAL_INSTALL_DIR"/lib
+  cmd cp -p /usr/lib/x86_64-linux-gnu/libssl.so.1.1 "$FINAL_INSTALL_DIR"/lib/libssl.so
 
   trace "Entering create_startup_script @ = $@"
   pushd .
@@ -2026,7 +2095,7 @@ End-of-environment-setup-template
   cp $TMPFILE "$FINAL_INSTALL_DIR/source-me" || die "Unable to create environment script - cp failed"
 
   log Creating wrapper scripts in $TMPFILE
-  for exe in melt ffmpeg ffplay ffprobe glaxnimate; do
+  for exe in melt ffmpeg ffplay ffprobe glaxnimate whisper-cli; do
     cat > $TMPFILE <<End-of-exe-wrapper
 #!/bin/sh
 # Set up environment
