@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2019 Meltytech, LLC
+ * Copyright (c) 2013-2025 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,16 +17,19 @@
 
 #include "customprofiledialog.h"
 #include "ui_customprofiledialog.h"
+
 #include "mltcontroller.h"
 #include "settings.h"
 #include "util.h"
-#include <QDir>
-#include <QDesktopServices>
 
-CustomProfileDialog::CustomProfileDialog(QWidget *parent) :
-    QDialog(parent),
-    ui(new Ui::CustomProfileDialog),
-    m_fps(0.0)
+#include <QDesktopServices>
+#include <QDir>
+#include <QRegularExpression>
+
+CustomProfileDialog::CustomProfileDialog(QWidget *parent)
+    : QDialog(parent)
+    , ui(new Ui::CustomProfileDialog)
+    , m_fps(0.0)
 {
     ui->setupUi(this);
     ui->widthSpinner->setValue(MLT.profile().width());
@@ -35,7 +38,17 @@ CustomProfileDialog::CustomProfileDialog(QWidget *parent) :
     ui->aspectDenSpinner->setValue(MLT.profile().display_aspect_den());
     ui->fpsSpinner->setValue(MLT.profile().fps());
     ui->scanModeCombo->setCurrentIndex(MLT.profile().progressive());
-    ui->colorspaceCombo->setCurrentIndex(MLT.profile().colorspace() == 709);
+    switch (MLT.profile().colorspace()) {
+    case 601:
+        ui->colorspaceCombo->setCurrentIndex(0);
+        break;
+    case 2020:
+        ui->colorspaceCombo->setCurrentIndex(2);
+        break;
+    default:
+        ui->colorspaceCombo->setCurrentIndex(1);
+        break;
+    }
 }
 
 CustomProfileDialog::~CustomProfileDialog()
@@ -45,7 +58,11 @@ CustomProfileDialog::~CustomProfileDialog()
 
 QString CustomProfileDialog::profileName() const
 {
-    return ui->nameEdit->text();
+    // Replace characters that are not allowed in Windows file names
+    QString filename = ui->nameEdit->text();
+    static QRegularExpression re("[" + QRegularExpression::escape("\\/:*?\"<>|") + "]");
+    filename = filename.replace(re, QStringLiteral("_"));
+    return filename;
 }
 
 void CustomProfileDialog::on_buttonBox_accepted()
@@ -54,30 +71,27 @@ void CustomProfileDialog::on_buttonBox_accepted()
     MLT.profile().set_width(ui->widthSpinner->value());
     MLT.profile().set_height(ui->heightSpinner->value());
     MLT.profile().set_display_aspect(ui->aspectNumSpinner->value(), ui->aspectDenSpinner->value());
-    int sar_num = ui->heightSpinner->value() * ui->aspectNumSpinner->value() / ui->aspectDenSpinner->value();//1024
-    int sar_den = ui->widthSpinner->value(); //720
-    if (sar_num == sar_den)
-        sar_num = sar_den = 1;
-    MLT.profile().set_sample_aspect(sar_num, sar_den);
-    switch (int(ui->fpsSpinner->value() * 10)) {
-    case 239:
-        MLT.profile().set_frame_rate(24000, 1001);
+    QSize sar(ui->aspectNumSpinner->value() * ui->heightSpinner->value(),
+              ui->aspectDenSpinner->value() * ui->widthSpinner->value());
+    auto gcd = Util::greatestCommonDivisor(sar.width(), sar.height());
+    MLT.profile().set_sample_aspect(sar.width() / gcd, sar.height() / gcd);
+    int numerator, denominator;
+    Util::normalizeFrameRate(ui->fpsSpinner->value(), numerator, denominator);
+    MLT.profile().set_frame_rate(numerator, denominator);
+    MLT.profile().set_progressive(ui->scanModeCombo->currentIndex());
+    switch (ui->colorspaceCombo->currentIndex()) {
+    case 0:
+        MLT.profile().set_colorspace(601);
         break;
-    case 299:
-        MLT.profile().set_frame_rate(30000, 1001);
-        break;
-    case 479:
-        MLT.profile().set_frame_rate(48000, 1001);
-        break;
-    case 599:
-        MLT.profile().set_frame_rate(60000, 1001);
+    case 2:
+        MLT.profile().set_colorspace(2020);
         break;
     default:
-        MLT.profile().set_frame_rate(ui->fpsSpinner->value() * 1000, 1000);
+        MLT.profile().set_colorspace(709);
         break;
     }
-    MLT.profile().set_progressive(ui->scanModeCombo->currentIndex());
-    MLT.profile().set_colorspace((ui->colorspaceCombo->currentIndex() == 1)? 709 : 601);
+    MLT.updatePreviewProfile();
+    MLT.setPreviewScale(Settings.playerPreviewScale());
 
     // Save it to a file
     if (!ui->nameEdit->text().isEmpty()) {
@@ -100,7 +114,7 @@ void CustomProfileDialog::on_buttonBox_accepted()
         p.set("colorspace", MLT.profile().colorspace());
         p.set("frame_rate_num", MLT.profile().frame_rate_num());
         p.set("frame_rate_den", MLT.profile().frame_rate_den());
-        p.save(dir.filePath(ui->nameEdit->text()).toUtf8().constData());
+        p.save(dir.filePath(profileName()).toUtf8().constData());
     }
 }
 
@@ -131,8 +145,27 @@ void CustomProfileDialog::on_fpsSpinner_editingFinished()
     }
 }
 
-void CustomProfileDialog::on_fpsComboBox_activated(const QString &arg1)
+void CustomProfileDialog::on_fpsComboBox_textActivated(const QString &arg1)
 {
-    if (!arg1.isEmpty())
-        ui->fpsSpinner->setValue(arg1.toDouble());
+    if (arg1.isEmpty())
+        return;
+    ui->fpsSpinner->setValue(arg1.toDouble());
+}
+
+void CustomProfileDialog::on_resolutionComboBox_textActivated(const QString &arg1)
+{
+    if (arg1.isEmpty())
+        return;
+    auto parts = arg1.split(' ');
+    ui->widthSpinner->setValue(parts[0].toInt());
+    ui->heightSpinner->setValue(parts[2].toInt());
+}
+
+void CustomProfileDialog::on_aspectRatioComboBox_textActivated(const QString &arg1)
+{
+    if (arg1.isEmpty())
+        return;
+    auto parts = arg1.split(' ')[0].split(':');
+    ui->aspectNumSpinner->setValue(parts[0].toInt());
+    ui->aspectDenSpinner->setValue(parts[1].toInt());
 }
