@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2021 Meltytech, LLC
+ * Copyright (c) 2012-2024 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,29 +16,35 @@
  */
 
 #include "meltjob.h"
-#include <QFile>
-#include <QFileInfo>
-#include <QDir>
-#include <QIODevice>
-#include <QApplication>
-#include <QAction>
-#include <QDialog>
-#include <QDir>
-#include <QTimer>
-#include <Logger.h>
-#include "mainwindow.h"
+
+#include "Logger.h"
 #include "dialogs/textviewerdialog.h"
+#include "mainwindow.h"
 #include "util.h"
 
-MeltJob::MeltJob(const QString& name, const QString& xml, int frameRateNum, int frameRateDen)
-    : AbstractJob(name)
+#include <QAction>
+#include <QApplication>
+#include <QDialog>
+#include <QDir>
+#include <QFile>
+#include <QFileInfo>
+#include <QIODevice>
+#include <QTimer>
+
+MeltJob::MeltJob(const QString &name,
+                 const QString &xml,
+                 int frameRateNum,
+                 int frameRateDen,
+                 QThread::Priority priority)
+    : AbstractJob(name, priority)
     , m_isStreaming(false)
     , m_previousPercent(0)
     , m_currentFrame(0)
     , m_useMultiConsumer(false)
 {
+    setTarget(name);
     if (!xml.isEmpty()) {
-        QAction* action = new QAction(tr("View XML"), this);
+        QAction *action = new QAction(tr("View XML"), this);
         action->setToolTip(tr("View the MLT XML for this job"));
         connect(action, SIGNAL(triggered()), this, SLOT(onViewXmlTriggered()));
         m_standardActions << action;
@@ -48,11 +54,17 @@ MeltJob::MeltJob(const QString& name, const QString& xml, int frameRateNum, int
         m_xml->close();
     } else {
         // Not an EncodeJob
-        QAction* action = new QAction(tr("Open"), this);
+        QAction *action = new QAction(tr("Open"), this);
+        action->setData("Open");
         action->setToolTip(tr("Open the output file in the Shotcut player"));
         connect(action, SIGNAL(triggered()), this, SLOT(onOpenTiggered()));
         m_successActions << action;
-    
+
+        action = new QAction(tr("Show In Folder"), this);
+        action->setToolTip(tr("Show In Files"));
+        connect(action, SIGNAL(triggered()), this, SLOT(onShowInFilesTriggered()));
+        m_successActions << action;
+
         action = new QAction(tr("Show In Folder"), this);
         action->setToolTip(tr("Show In Folder"));
         connect(action, SIGNAL(triggered()), this, SLOT(onShowFolderTriggered()));
@@ -72,7 +84,22 @@ void MeltJob::onShowFolderTriggered()
     Util::showInFolder(objectName());
 }
 
-MeltJob::MeltJob(const QString& name, const QStringList& args, int frameRateNum, int frameRateDen)
+void MeltJob::onShowInFilesTriggered()
+{
+    MAIN.showInFiles(objectName());
+}
+
+MeltJob::MeltJob(const QString &name,
+                 const QString &xml,
+                 const QStringList &args,
+                 int frameRateNum,
+                 int frameRateDen)
+    : MeltJob(name, xml, frameRateNum, frameRateDen)
+{
+    m_args = args;
+}
+
+MeltJob::MeltJob(const QString &name, const QStringList &args, int frameRateNum, int frameRateDen)
     : MeltJob(name, QString(), frameRateNum, frameRateDen)
 {
     m_args = args;
@@ -89,9 +116,7 @@ void MeltJob::start()
         AbstractJob::start();
         LOG_ERROR() << "the job XML is empty!";
         appendToLog("Error: the job XML is empty!\n");
-        QTimer::singleShot(0, this, [=]() {
-            emit finished(this, false);
-        });
+        QTimer::singleShot(0, this, [=]() { emit finished(this, false); });
         return;
     }
     QString shotcutPath = qApp->applicationDirPath();
@@ -105,20 +130,23 @@ void MeltJob::start()
     args << "-verbose";
     args << "-progress2";
     args << "-abort";
+    if (!m_xml.isNull()) {
+        if (m_useMultiConsumer) {
+            args << "xml:" + QUrl::toPercentEncoding(xmlPath()) + "?multi:1";
+        } else {
+            args << "xml:" + QUrl::toPercentEncoding(xmlPath());
+        }
+    }
     if (m_args.size() > 0) {
         args.append(m_args);
-    } else if (m_useMultiConsumer) {
-        args << "xml:" + QUrl::toPercentEncoding(xmlPath()) + "?multi:1";
-    } else {
-        args << "xml:" + QUrl::toPercentEncoding(xmlPath());
     }
     if (m_in > -1) {
-        args << QString("in=%1").arg(m_in);
+        args << QStringLiteral("in=%1").arg(m_in);
     }
     if (m_out > -1) {
-        args << QString("out=%1").arg(m_out);
+        args << QStringLiteral("out=%1").arg(m_out);
     }
-    LOG_DEBUG() << meltPath.absoluteFilePath()  + " " + args.join(' ');
+    LOG_DEBUG() << meltPath.absoluteFilePath() + " " + args.join(' ');
 #ifndef Q_OS_MAC
     QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
     // These environment variables fix rich text rendering for high DPI
@@ -128,15 +156,10 @@ void MeltJob::start()
     setProcessEnvironment(env);
 #endif
 #ifdef Q_OS_WIN
-    if (m_isStreaming) args << "-getc";
-    QProcess::start(meltPath.absoluteFilePath(), args);
-#else
-    args.prepend(meltPath.absoluteFilePath());
-    args.prepend("3");
-    args.prepend("-n");
-    QProcess::start("nice", args);
+    if (m_isStreaming)
+        args << "-getc";
 #endif
-    AbstractJob::start();
+    AbstractJob::start(meltPath.absoluteFilePath(), args);
 }
 
 QString MeltJob::xml()
@@ -165,7 +188,7 @@ void MeltJob::setInAndOut(int in, int out)
 
 void MeltJob::onViewXmlTriggered()
 {
-    TextViewerDialog dialog(&MAIN);
+    TextViewerDialog dialog(&MAIN, true);
     dialog.setWindowTitle(tr("MLT XML"));
     dialog.setText(xml());
     dialog.exec();
@@ -185,13 +208,12 @@ void MeltJob::onReadyRead()
         index = msg.indexOf("percentage:");
         if (index > -1) {
             int percent = msg.mid(index + 11).toInt();
-            if (percent != m_previousPercent) {
+            if (percent > 0 && percent != m_previousPercent) {
                 emit progressUpdated(m_item, percent);
                 QCoreApplication::processEvents();
                 m_previousPercent = percent;
             }
-        }
-        else {
+        } else {
             appendToLog(msg);
         }
     } while (!msg.isEmpty());
