@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2024 Meltytech, LLC
+ * Copyright (c) 2011-2025 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,16 +18,17 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
+#include "mltcontroller.h"
+#include "mltxmlchecker.h"
+
+#include <QDateTime>
 #include <QMainWindow>
 #include <QMutex>
-#include <QTimer>
-#include <QUrl>
 #include <QNetworkAccessManager>
 #include <QScopedPointer>
 #include <QSharedPointer>
-#include <QDateTime>
-#include "mltcontroller.h"
-#include "mltxmlchecker.h"
+#include <QTimer>
+#include <QUrl>
 
 #define EXIT_RESTART (42)
 #define EXIT_RESET (43)
@@ -44,6 +45,7 @@ class QUndoStack;
 class QActionGroup;
 class FilterController;
 class ScopeController;
+class FilesDock;
 class FiltersDock;
 class TimelineDock;
 class AutoSaveFile;
@@ -58,15 +60,7 @@ class MainWindow : public QMainWindow
     Q_OBJECT
 
 public:
-    enum LayoutMode {
-        Custom = 0,
-        Logging,
-        Editing,
-        Effects,
-        Color,
-        Audio,
-        PlayerOnly
-    };
+    enum LayoutMode { Custom = 0, Logging, Editing, Effects, Color, Audio, PlayerOnly };
 
     static MainWindow &singleton();
     ~MainWindow();
@@ -76,14 +70,9 @@ public:
     QUndoStack *undoStack() const;
     bool saveXML(const QString &filename, bool withRelativePaths = true);
     static void changeTheme(const QString &theme);
-    PlaylistDock *playlistDock() const
-    {
-        return m_playlistDock;
-    }
-    FilterController *filterController() const
-    {
-        return m_filterController;
-    }
+    PlaylistDock *playlistDock() const { return m_playlistDock; }
+    TimelineDock *timelineDock() const { return m_timelineDock; }
+    FilterController *filterController() const { return m_filterController; }
     Mlt::Playlist *playlist() const;
     bool isPlaylistValid() const;
     Mlt::Producer *multitrack() const;
@@ -92,27 +81,21 @@ public:
     void setFullScreen(bool isFullScreen);
     QString untitledFileName() const;
     void setProfile(const QString &profile_name);
-    QString fileName() const
-    {
-        return m_currentFile;
-    }
+    QString fileName() const { return m_currentFile; }
     bool isSourceClipMyProject(QString resource = MLT.resource(), bool withDialog = true);
     bool keyframesDockIsVisible() const;
 
     void keyPressEvent(QKeyEvent *);
     void keyReleaseEvent(QKeyEvent *);
     void hideSetDataDirectory();
-    QMenu *customProfileMenu() const
-    {
-        return m_customProfileMenu;
-    }
+    QMenu *customProfileMenu() const { return m_customProfileMenu; }
     QAction *actionAddCustomProfile() const;
     QAction *actionProfileRemove() const;
-    QActionGroup *profileGroup() const
-    {
-        return m_profileGroup;
-    }
-    void buildVideoModeMenu(QMenu *topMenu, QMenu *&customMenu, QActionGroup *group, QAction *addAction,
+    QActionGroup *profileGroup() const { return m_profileGroup; }
+    void buildVideoModeMenu(QMenu *topMenu,
+                            QMenu *&customMenu,
+                            QActionGroup *group,
+                            QAction *addAction,
                             QAction *removeAction);
     void newProject(const QString &filename, bool isProjectFolder = false);
     void addCustomProfile(const QString &name, QMenu *menu, QAction *action, QActionGroup *group);
@@ -120,15 +103,14 @@ public:
     QUuid timelineClipUuid(int trackIndex, int clipIndex);
     void replaceInTimeline(const QUuid &uuid, Mlt::Producer &producer);
     void replaceAllByHash(const QString &hash, Mlt::Producer &producer, bool isProxy = false);
-    bool isClipboardNewer() const
-    {
-        return m_clipboardUpdatedAt > m_sourceUpdatedAt;
-    }
+    bool isClipboardNewer() const { return m_clipboardUpdatedAt > m_sourceUpdatedAt; }
     int mltIndexForTrack(int trackIndex) const;
     int bottomVideoTrackIndex() const;
     void cropSource(const QRectF &rect);
     void getMarkerRange(int position, int *start, int *end);
     void getSelectionRange(int *start, int *end);
+    Mlt::Playlist *binPlaylist();
+    void showInFiles(const QString &filePath);
 
 signals:
     void audioChannelsChanged();
@@ -171,6 +153,7 @@ private:
     void writeSettings();
     void configureVideoWidget();
     void setCurrentFile(const QString &filename);
+    void updateWindowTitle();
     void changeAudioChannels(bool checked, int channels);
     void changeDeinterlacer(bool checked, const char *method);
     void changeInterpolation(bool checked, const char *method);
@@ -187,6 +170,7 @@ private:
     void backup();
     void backupPeriodically();
     bool confirmProfileChange();
+    bool confirmRestartExternalMonitor();
 
     Ui::MainWindow *ui;
     Player *m_player;
@@ -202,6 +186,7 @@ private:
     QDockWidget *m_historyDock;
     QActionGroup *m_profileGroup;
     QActionGroup *m_externalGroup;
+    QActionGroup *m_decklinkGammaGroup{nullptr};
     QActionGroup *m_keyerGroup;
     QActionGroup *m_layoutGroup;
     QActionGroup *m_previewScaleGroup;
@@ -209,6 +194,7 @@ private:
     FilterController *m_filterController;
     ScopeController *m_scopeController;
     QMenu *m_customProfileMenu;
+    QMenu *m_decklinkGammaMenu{nullptr};
     QMenu *m_keyerMenu;
     QStringList m_multipleFiles;
     bool m_multipleFilesLoading;
@@ -228,11 +214,14 @@ private:
     NotesDock *m_notesDock;
     SubtitlesDock *m_subtitlesDock;
     std::unique_ptr<QWidget> m_producerWidget;
+    FilesDock *m_filesDock;
 
 public slots:
     bool isCompatibleWithGpuMode(MltXmlChecker &checker);
     bool isXmlRepaired(MltXmlChecker &checker, QString &fileName);
-    bool open(QString url, const Mlt::Properties * = nullptr, bool play = true,
+    bool open(QString url,
+              const Mlt::Properties * = nullptr,
+              bool play = true,
               bool skipConvert = false);
     void openMultiple(const QStringList &paths);
     void openMultiple(const QList<QUrl> &urls);
@@ -241,7 +230,8 @@ public slots:
     void hideProducer();
     void closeProducer();
     void showStatusMessage(QAction *action, int timeoutSeconds = 5);
-    void showStatusMessage(const QString &message, int timeoutSeconds = 5,
+    void showStatusMessage(const QString &message,
+                           int timeoutSeconds = 5,
                            QPalette::ColorRole role = QPalette::ToolTipBase);
     void onStatusMessageClicked();
     void seekPlaylist(int start);
@@ -252,8 +242,10 @@ public slots:
     void onGpuNotSupported();
     void onShuttle(float x);
     void onPropertiesDockTriggered(bool checked = true);
+    void onFiltersDockTriggered(bool checked = true);
     bool on_actionSave_triggered();
     void onCreateOrEditFilterOnOutput(Mlt::Filter *filter, const QStringList &key_properties);
+    void showSettingsMenu() const;
 
 private slots:
     void showUpgradePrompt();
@@ -268,11 +260,11 @@ private slots:
     void onPlaylistDockTriggered(bool checked = true);
     void onTimelineDockTriggered(bool checked = true);
     void onHistoryDockTriggered(bool checked = true);
-    void onFiltersDockTriggered(bool checked = true);
     void onKeyframesDockTriggered(bool checked = true);
     void onMarkersDockTriggered(bool = true);
     void onNotesDockTriggered(bool = true);
     void onSubtitlesDockTriggered(bool = true);
+    void onFilesDockTriggered(bool = true);
     void onPlaylistCreated();
     void onPlaylistLoaded();
     void onPlaylistCleared();
@@ -312,6 +304,7 @@ private slots:
     void on_actionJack_triggered(bool checked);
     void on_actionGPU_triggered(bool checked);
     void onExternalTriggered(QAction *);
+    void onDecklinkGammaTriggered(QAction *);
     void onKeyerTriggered(QAction *);
     void onProfileTriggered(QAction *);
     void onProfileChanged();
@@ -320,6 +313,7 @@ private slots:
     void processSingleFile();
     void onLanguageTriggered(QAction *);
     void on_actionSystemTheme_triggered();
+    void on_actionSystemFusion_triggered();
     void on_actionFusionDark_triggered();
     void on_actionJobPriorityLow_triggered();
     void on_actionJobPriorityNormal_triggered();
@@ -332,6 +326,7 @@ private slots:
     void on_menuExternal_aboutToShow();
     void on_actionUpgrade_triggered();
     void on_actionOpenXML_triggered();
+    void on_actionShowProjectFolder_triggered();
     void onAutosaveTimeout();
     void onFocusChanged(QWidget *old, QWidget *now) const;
     void onFocusObjectChanged(QObject *obj) const;
@@ -382,6 +377,7 @@ private slots:
     void on_actionPreview360_triggered(bool checked);
     void on_actionPreview540_triggered(bool checked);
     void on_actionPreview720_triggered(bool checked);
+    void on_actionPreview1080_triggered(bool checked);
     void on_actionTopics_triggered();
     void on_actionSync_triggered();
     void on_actionUseProxy_triggered(bool checked);
@@ -399,6 +395,7 @@ private slots:
     void on_actionAudioVideoDevice_triggered();
     void on_actionReset_triggered();
     void on_actionBackupSave_triggered();
+    void on_actionPauseAfterSeek_triggered(bool checked);
 };
 
 #define MAIN MainWindow::singleton()
