@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2019 Meltytech, LLC
+ * Copyright (c) 2016-2025 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,41 +25,59 @@ class PlaylistIconView : public QAbstractItemView
     Q_OBJECT
 public:
     PlaylistIconView(QWidget *parent);
+    void resetMultiSelect();
+    void setIconRole(int role);
 
     QRect visualRect(const QModelIndex &index) const Q_DECL_OVERRIDE;
     void scrollTo(const QModelIndex &index, ScrollHint hint = EnsureVisible) Q_DECL_OVERRIDE;
     QModelIndex indexAt(const QPoint &point) const Q_DECL_OVERRIDE;
-    QModelIndex moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers) Q_DECL_OVERRIDE;
+    QModelIndex moveCursor(CursorAction cursorAction,
+                           Qt::KeyboardModifiers modifiers) Q_DECL_OVERRIDE;
     int horizontalOffset() const Q_DECL_OVERRIDE;
     int verticalOffset() const Q_DECL_OVERRIDE;
     bool isIndexHidden(const QModelIndex &index) const Q_DECL_OVERRIDE;
-    void setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags command) Q_DECL_OVERRIDE;
+    void setSelection(const QRect &rect,
+                      QItemSelectionModel::SelectionFlags command) Q_DECL_OVERRIDE;
     QRegion visualRegionForSelection(const QItemSelection &selection) const Q_DECL_OVERRIDE;
     void currentChanged(const QModelIndex &current, const QModelIndex &previous) Q_DECL_OVERRIDE;
 
-    void paintEvent(QPaintEvent*) Q_DECL_OVERRIDE;
+    void paintEvent(QPaintEvent *) Q_DECL_OVERRIDE;
     void mouseReleaseEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
     void dragMoveEvent(QDragMoveEvent *e) Q_DECL_OVERRIDE;
     void dragLeaveEvent(QDragLeaveEvent *e) Q_DECL_OVERRIDE;
     void dropEvent(QDropEvent *e) Q_DECL_OVERRIDE;
     void resizeEvent(QResizeEvent *event) Q_DECL_OVERRIDE;
-    void setModel(QAbstractItemModel* model) Q_DECL_OVERRIDE;
-    void keyPressEvent(QKeyEvent* event) Q_DECL_OVERRIDE;
+    void setModel(QAbstractItemModel *model) Q_DECL_OVERRIDE;
+    void keyPressEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
+    void keyReleaseEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
 
     void rowsInserted(const QModelIndex &parent, int start, int end) Q_DECL_OVERRIDE;
     void rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end) Q_DECL_OVERRIDE;
-    void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles = QVector<int>()) Q_DECL_OVERRIDE;
+    void dataChanged(const QModelIndex &topLeft,
+                     const QModelIndex &bottomRight,
+                     const QVector<int> &roles = QVector<int>()) Q_DECL_OVERRIDE;
 
-private slots:
+public slots:
     void updateSizes();
 
+protected slots:
+    void selectionChanged(const QItemSelection &selected,
+                          const QItemSelection &deselected) Q_DECL_OVERRIDE;
+
 private:
     int rowWidth() const;
-    QAbstractItemView::DropIndicatorPosition position(const QPoint &pos, const QRect &rect, const QModelIndex &index) const;
+    QAbstractItemView::DropIndicatorPosition position(const QPoint &pos,
+                                                      const QRect &rect,
+                                                      const QModelIndex &index) const;
+    QRect _visualRect(const QModelIndex &index) const;
 
     QSize m_gridSize;
     QPoint m_draggingOverPos;
     int m_itemsPerRow;
+    bool m_isToggleSelect{false};
+    bool m_isRangeSelect{false};
+    QModelIndex m_pendingSelect;
+    int m_iconRole;
 };
 
 #endif
